from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class HeatFluxUnits(Enum):
        """
            HeatFluxUnits enumeration
        """
        
        WattPerSquareMeter = 'WattPerSquareMeter'
        """
            
        """
        
        WattPerSquareInch = 'WattPerSquareInch'
        """
            
        """
        
        WattPerSquareFoot = 'WattPerSquareFoot'
        """
            
        """
        
        BtuPerSecondSquareInch = 'BtuPerSecondSquareInch'
        """
            
        """
        
        BtuPerSecondSquareFoot = 'BtuPerSecondSquareFoot'
        """
            
        """
        
        BtuPerMinuteSquareFoot = 'BtuPerMinuteSquareFoot'
        """
            
        """
        
        BtuPerHourSquareFoot = 'BtuPerHourSquareFoot'
        """
            
        """
        
        CaloriePerSecondSquareCentimeter = 'CaloriePerSecondSquareCentimeter'
        """
            
        """
        
        KilocaloriePerHourSquareMeter = 'KilocaloriePerHourSquareMeter'
        """
            
        """
        
        PoundForcePerFootSecond = 'PoundForcePerFootSecond'
        """
            
        """
        
        PoundPerSecondCubed = 'PoundPerSecondCubed'
        """
            
        """
        
        NanowattPerSquareMeter = 'NanowattPerSquareMeter'
        """
            
        """
        
        MicrowattPerSquareMeter = 'MicrowattPerSquareMeter'
        """
            
        """
        
        MilliwattPerSquareMeter = 'MilliwattPerSquareMeter'
        """
            
        """
        
        CentiwattPerSquareMeter = 'CentiwattPerSquareMeter'
        """
            
        """
        
        DeciwattPerSquareMeter = 'DeciwattPerSquareMeter'
        """
            
        """
        
        KilowattPerSquareMeter = 'KilowattPerSquareMeter'
        """
            
        """
        
        KilocaloriePerSecondSquareCentimeter = 'KilocaloriePerSecondSquareCentimeter'
        """
            
        """
        

class HeatFluxDto:
    """
    A DTO representation of a HeatFlux

    Attributes:
        value (float): The value of the HeatFlux.
        unit (HeatFluxUnits): The specific unit that the HeatFlux value is representing.
    """

    def __init__(self, value: float, unit: HeatFluxUnits):
        """
        Create a new DTO representation of a HeatFlux

        Parameters:
            value (float): The value of the HeatFlux.
            unit (HeatFluxUnits): The specific unit that the HeatFlux value is representing.
        """
        self.value: float = value
        """
        The value of the HeatFlux
        """
        self.unit: HeatFluxUnits = unit
        """
        The specific unit that the HeatFlux value is representing
        """

    def to_json(self):
        """
        Get a HeatFlux DTO JSON object representing the current unit.

        :return: JSON object represents HeatFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of HeatFlux DTO from a json representation.

        :param data: The HeatFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerSquareMeter"}
        :return: A new instance of HeatFluxDto.
        :rtype: HeatFluxDto
        """
        return HeatFluxDto(value=data["value"], unit=HeatFluxUnits(data["unit"]))


class HeatFlux(AbstractMeasure):
    """
    Heat flux is the flow of energy per unit of area per unit of time

    Args:
        value (float): The value.
        from_unit (HeatFluxUnits): The HeatFlux unit to create from, The default unit is WattPerSquareMeter
    """
    def __init__(self, value: float, from_unit: HeatFluxUnits = HeatFluxUnits.WattPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_square_meter = None
        
        self.__watts_per_square_inch = None
        
        self.__watts_per_square_foot = None
        
        self.__btus_per_second_square_inch = None
        
        self.__btus_per_second_square_foot = None
        
        self.__btus_per_minute_square_foot = None
        
        self.__btus_per_hour_square_foot = None
        
        self.__calories_per_second_square_centimeter = None
        
        self.__kilocalories_per_hour_square_meter = None
        
        self.__pounds_force_per_foot_second = None
        
        self.__pounds_per_second_cubed = None
        
        self.__nanowatts_per_square_meter = None
        
        self.__microwatts_per_square_meter = None
        
        self.__milliwatts_per_square_meter = None
        
        self.__centiwatts_per_square_meter = None
        
        self.__deciwatts_per_square_meter = None
        
        self.__kilowatts_per_square_meter = None
        
        self.__kilocalories_per_second_square_centimeter = None
        

    def convert(self, unit: HeatFluxUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: HeatFluxUnits = HeatFluxUnits.WattPerSquareMeter) -> HeatFluxDto:
        """
        Get a new instance of HeatFlux DTO representing the current unit.

        :param hold_in_unit: The specific HeatFlux unit to store the HeatFlux value in the DTO representation.
        :type hold_in_unit: HeatFluxUnits
        :return: A new instance of HeatFluxDto.
        :rtype: HeatFluxDto
        """
        return HeatFluxDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: HeatFluxUnits = HeatFluxUnits.WattPerSquareMeter):
        """
        Get a HeatFlux DTO JSON object representing the current unit.

        :param hold_in_unit: The specific HeatFlux unit to store the HeatFlux value in the DTO representation.
        :type hold_in_unit: HeatFluxUnits
        :return: JSON object represents HeatFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(heat_flux_dto: HeatFluxDto):
        """
        Obtain a new instance of HeatFlux from a DTO unit object.

        :param heat_flux_dto: The HeatFlux DTO representation.
        :type heat_flux_dto: HeatFluxDto
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(heat_flux_dto.value, heat_flux_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of HeatFlux from a DTO unit json representation.

        :param data: The HeatFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerSquareMeter"}
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux.from_dto(HeatFluxDto.from_json(data))

    def __convert_from_base(self, from_unit: HeatFluxUnits) -> float:
        value = self._value
        
        if from_unit == HeatFluxUnits.WattPerSquareMeter:
            return (value)
        
        if from_unit == HeatFluxUnits.WattPerSquareInch:
            return (value / 1.5500031e3)
        
        if from_unit == HeatFluxUnits.WattPerSquareFoot:
            return (value / 1.07639e1)
        
        if from_unit == HeatFluxUnits.BtuPerSecondSquareInch:
            return (value / 1.63533984e6)
        
        if from_unit == HeatFluxUnits.BtuPerSecondSquareFoot:
            return (value / 1.13565267e4)
        
        if from_unit == HeatFluxUnits.BtuPerMinuteSquareFoot:
            return (value / 1.89275445e2)
        
        if from_unit == HeatFluxUnits.BtuPerHourSquareFoot:
            return (value / 3.15459075)
        
        if from_unit == HeatFluxUnits.CaloriePerSecondSquareCentimeter:
            return (value / 4.1868e4)
        
        if from_unit == HeatFluxUnits.KilocaloriePerHourSquareMeter:
            return (value / 1.163)
        
        if from_unit == HeatFluxUnits.PoundForcePerFootSecond:
            return (value / 1.459390293720636e1)
        
        if from_unit == HeatFluxUnits.PoundPerSecondCubed:
            return (value / 4.5359237e-1)
        
        if from_unit == HeatFluxUnits.NanowattPerSquareMeter:
            return ((value) / 1e-09)
        
        if from_unit == HeatFluxUnits.MicrowattPerSquareMeter:
            return ((value) / 1e-06)
        
        if from_unit == HeatFluxUnits.MilliwattPerSquareMeter:
            return ((value) / 0.001)
        
        if from_unit == HeatFluxUnits.CentiwattPerSquareMeter:
            return ((value) / 0.01)
        
        if from_unit == HeatFluxUnits.DeciwattPerSquareMeter:
            return ((value) / 0.1)
        
        if from_unit == HeatFluxUnits.KilowattPerSquareMeter:
            return ((value) / 1000.0)
        
        if from_unit == HeatFluxUnits.KilocaloriePerSecondSquareCentimeter:
            return ((value / 4.1868e4) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: HeatFluxUnits) -> float:
        
        if to_unit == HeatFluxUnits.WattPerSquareMeter:
            return (value)
        
        if to_unit == HeatFluxUnits.WattPerSquareInch:
            return (value * 1.5500031e3)
        
        if to_unit == HeatFluxUnits.WattPerSquareFoot:
            return (value * 1.07639e1)
        
        if to_unit == HeatFluxUnits.BtuPerSecondSquareInch:
            return (value * 1.63533984e6)
        
        if to_unit == HeatFluxUnits.BtuPerSecondSquareFoot:
            return (value * 1.13565267e4)
        
        if to_unit == HeatFluxUnits.BtuPerMinuteSquareFoot:
            return (value * 1.89275445e2)
        
        if to_unit == HeatFluxUnits.BtuPerHourSquareFoot:
            return (value * 3.15459075)
        
        if to_unit == HeatFluxUnits.CaloriePerSecondSquareCentimeter:
            return (value * 4.1868e4)
        
        if to_unit == HeatFluxUnits.KilocaloriePerHourSquareMeter:
            return (value * 1.163)
        
        if to_unit == HeatFluxUnits.PoundForcePerFootSecond:
            return (value * 1.459390293720636e1)
        
        if to_unit == HeatFluxUnits.PoundPerSecondCubed:
            return (value * 4.5359237e-1)
        
        if to_unit == HeatFluxUnits.NanowattPerSquareMeter:
            return ((value) * 1e-09)
        
        if to_unit == HeatFluxUnits.MicrowattPerSquareMeter:
            return ((value) * 1e-06)
        
        if to_unit == HeatFluxUnits.MilliwattPerSquareMeter:
            return ((value) * 0.001)
        
        if to_unit == HeatFluxUnits.CentiwattPerSquareMeter:
            return ((value) * 0.01)
        
        if to_unit == HeatFluxUnits.DeciwattPerSquareMeter:
            return ((value) * 0.1)
        
        if to_unit == HeatFluxUnits.KilowattPerSquareMeter:
            return ((value) * 1000.0)
        
        if to_unit == HeatFluxUnits.KilocaloriePerSecondSquareCentimeter:
            return ((value * 4.1868e4) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_watts_per_square_meter(watts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in watts_per_square_meter.

        

        :param meters: The HeatFlux value in watts_per_square_meter.
        :type watts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(watts_per_square_meter, HeatFluxUnits.WattPerSquareMeter)

    
    @staticmethod
    def from_watts_per_square_inch(watts_per_square_inch: float):
        """
        Create a new instance of HeatFlux from a value in watts_per_square_inch.

        

        :param meters: The HeatFlux value in watts_per_square_inch.
        :type watts_per_square_inch: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(watts_per_square_inch, HeatFluxUnits.WattPerSquareInch)

    
    @staticmethod
    def from_watts_per_square_foot(watts_per_square_foot: float):
        """
        Create a new instance of HeatFlux from a value in watts_per_square_foot.

        

        :param meters: The HeatFlux value in watts_per_square_foot.
        :type watts_per_square_foot: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(watts_per_square_foot, HeatFluxUnits.WattPerSquareFoot)

    
    @staticmethod
    def from_btus_per_second_square_inch(btus_per_second_square_inch: float):
        """
        Create a new instance of HeatFlux from a value in btus_per_second_square_inch.

        

        :param meters: The HeatFlux value in btus_per_second_square_inch.
        :type btus_per_second_square_inch: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(btus_per_second_square_inch, HeatFluxUnits.BtuPerSecondSquareInch)

    
    @staticmethod
    def from_btus_per_second_square_foot(btus_per_second_square_foot: float):
        """
        Create a new instance of HeatFlux from a value in btus_per_second_square_foot.

        

        :param meters: The HeatFlux value in btus_per_second_square_foot.
        :type btus_per_second_square_foot: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(btus_per_second_square_foot, HeatFluxUnits.BtuPerSecondSquareFoot)

    
    @staticmethod
    def from_btus_per_minute_square_foot(btus_per_minute_square_foot: float):
        """
        Create a new instance of HeatFlux from a value in btus_per_minute_square_foot.

        

        :param meters: The HeatFlux value in btus_per_minute_square_foot.
        :type btus_per_minute_square_foot: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(btus_per_minute_square_foot, HeatFluxUnits.BtuPerMinuteSquareFoot)

    
    @staticmethod
    def from_btus_per_hour_square_foot(btus_per_hour_square_foot: float):
        """
        Create a new instance of HeatFlux from a value in btus_per_hour_square_foot.

        

        :param meters: The HeatFlux value in btus_per_hour_square_foot.
        :type btus_per_hour_square_foot: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(btus_per_hour_square_foot, HeatFluxUnits.BtuPerHourSquareFoot)

    
    @staticmethod
    def from_calories_per_second_square_centimeter(calories_per_second_square_centimeter: float):
        """
        Create a new instance of HeatFlux from a value in calories_per_second_square_centimeter.

        

        :param meters: The HeatFlux value in calories_per_second_square_centimeter.
        :type calories_per_second_square_centimeter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(calories_per_second_square_centimeter, HeatFluxUnits.CaloriePerSecondSquareCentimeter)

    
    @staticmethod
    def from_kilocalories_per_hour_square_meter(kilocalories_per_hour_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in kilocalories_per_hour_square_meter.

        

        :param meters: The HeatFlux value in kilocalories_per_hour_square_meter.
        :type kilocalories_per_hour_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(kilocalories_per_hour_square_meter, HeatFluxUnits.KilocaloriePerHourSquareMeter)

    
    @staticmethod
    def from_pounds_force_per_foot_second(pounds_force_per_foot_second: float):
        """
        Create a new instance of HeatFlux from a value in pounds_force_per_foot_second.

        

        :param meters: The HeatFlux value in pounds_force_per_foot_second.
        :type pounds_force_per_foot_second: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(pounds_force_per_foot_second, HeatFluxUnits.PoundForcePerFootSecond)

    
    @staticmethod
    def from_pounds_per_second_cubed(pounds_per_second_cubed: float):
        """
        Create a new instance of HeatFlux from a value in pounds_per_second_cubed.

        

        :param meters: The HeatFlux value in pounds_per_second_cubed.
        :type pounds_per_second_cubed: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(pounds_per_second_cubed, HeatFluxUnits.PoundPerSecondCubed)

    
    @staticmethod
    def from_nanowatts_per_square_meter(nanowatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in nanowatts_per_square_meter.

        

        :param meters: The HeatFlux value in nanowatts_per_square_meter.
        :type nanowatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(nanowatts_per_square_meter, HeatFluxUnits.NanowattPerSquareMeter)

    
    @staticmethod
    def from_microwatts_per_square_meter(microwatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in microwatts_per_square_meter.

        

        :param meters: The HeatFlux value in microwatts_per_square_meter.
        :type microwatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(microwatts_per_square_meter, HeatFluxUnits.MicrowattPerSquareMeter)

    
    @staticmethod
    def from_milliwatts_per_square_meter(milliwatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in milliwatts_per_square_meter.

        

        :param meters: The HeatFlux value in milliwatts_per_square_meter.
        :type milliwatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(milliwatts_per_square_meter, HeatFluxUnits.MilliwattPerSquareMeter)

    
    @staticmethod
    def from_centiwatts_per_square_meter(centiwatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in centiwatts_per_square_meter.

        

        :param meters: The HeatFlux value in centiwatts_per_square_meter.
        :type centiwatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(centiwatts_per_square_meter, HeatFluxUnits.CentiwattPerSquareMeter)

    
    @staticmethod
    def from_deciwatts_per_square_meter(deciwatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in deciwatts_per_square_meter.

        

        :param meters: The HeatFlux value in deciwatts_per_square_meter.
        :type deciwatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(deciwatts_per_square_meter, HeatFluxUnits.DeciwattPerSquareMeter)

    
    @staticmethod
    def from_kilowatts_per_square_meter(kilowatts_per_square_meter: float):
        """
        Create a new instance of HeatFlux from a value in kilowatts_per_square_meter.

        

        :param meters: The HeatFlux value in kilowatts_per_square_meter.
        :type kilowatts_per_square_meter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(kilowatts_per_square_meter, HeatFluxUnits.KilowattPerSquareMeter)

    
    @staticmethod
    def from_kilocalories_per_second_square_centimeter(kilocalories_per_second_square_centimeter: float):
        """
        Create a new instance of HeatFlux from a value in kilocalories_per_second_square_centimeter.

        

        :param meters: The HeatFlux value in kilocalories_per_second_square_centimeter.
        :type kilocalories_per_second_square_centimeter: float
        :return: A new instance of HeatFlux.
        :rtype: HeatFlux
        """
        return HeatFlux(kilocalories_per_second_square_centimeter, HeatFluxUnits.KilocaloriePerSecondSquareCentimeter)

    
    @property
    def watts_per_square_meter(self) -> float:
        """
        
        """
        if self.__watts_per_square_meter != None:
            return self.__watts_per_square_meter
        self.__watts_per_square_meter = self.__convert_from_base(HeatFluxUnits.WattPerSquareMeter)
        return self.__watts_per_square_meter

    
    @property
    def watts_per_square_inch(self) -> float:
        """
        
        """
        if self.__watts_per_square_inch != None:
            return self.__watts_per_square_inch
        self.__watts_per_square_inch = self.__convert_from_base(HeatFluxUnits.WattPerSquareInch)
        return self.__watts_per_square_inch

    
    @property
    def watts_per_square_foot(self) -> float:
        """
        
        """
        if self.__watts_per_square_foot != None:
            return self.__watts_per_square_foot
        self.__watts_per_square_foot = self.__convert_from_base(HeatFluxUnits.WattPerSquareFoot)
        return self.__watts_per_square_foot

    
    @property
    def btus_per_second_square_inch(self) -> float:
        """
        
        """
        if self.__btus_per_second_square_inch != None:
            return self.__btus_per_second_square_inch
        self.__btus_per_second_square_inch = self.__convert_from_base(HeatFluxUnits.BtuPerSecondSquareInch)
        return self.__btus_per_second_square_inch

    
    @property
    def btus_per_second_square_foot(self) -> float:
        """
        
        """
        if self.__btus_per_second_square_foot != None:
            return self.__btus_per_second_square_foot
        self.__btus_per_second_square_foot = self.__convert_from_base(HeatFluxUnits.BtuPerSecondSquareFoot)
        return self.__btus_per_second_square_foot

    
    @property
    def btus_per_minute_square_foot(self) -> float:
        """
        
        """
        if self.__btus_per_minute_square_foot != None:
            return self.__btus_per_minute_square_foot
        self.__btus_per_minute_square_foot = self.__convert_from_base(HeatFluxUnits.BtuPerMinuteSquareFoot)
        return self.__btus_per_minute_square_foot

    
    @property
    def btus_per_hour_square_foot(self) -> float:
        """
        
        """
        if self.__btus_per_hour_square_foot != None:
            return self.__btus_per_hour_square_foot
        self.__btus_per_hour_square_foot = self.__convert_from_base(HeatFluxUnits.BtuPerHourSquareFoot)
        return self.__btus_per_hour_square_foot

    
    @property
    def calories_per_second_square_centimeter(self) -> float:
        """
        
        """
        if self.__calories_per_second_square_centimeter != None:
            return self.__calories_per_second_square_centimeter
        self.__calories_per_second_square_centimeter = self.__convert_from_base(HeatFluxUnits.CaloriePerSecondSquareCentimeter)
        return self.__calories_per_second_square_centimeter

    
    @property
    def kilocalories_per_hour_square_meter(self) -> float:
        """
        
        """
        if self.__kilocalories_per_hour_square_meter != None:
            return self.__kilocalories_per_hour_square_meter
        self.__kilocalories_per_hour_square_meter = self.__convert_from_base(HeatFluxUnits.KilocaloriePerHourSquareMeter)
        return self.__kilocalories_per_hour_square_meter

    
    @property
    def pounds_force_per_foot_second(self) -> float:
        """
        
        """
        if self.__pounds_force_per_foot_second != None:
            return self.__pounds_force_per_foot_second
        self.__pounds_force_per_foot_second = self.__convert_from_base(HeatFluxUnits.PoundForcePerFootSecond)
        return self.__pounds_force_per_foot_second

    
    @property
    def pounds_per_second_cubed(self) -> float:
        """
        
        """
        if self.__pounds_per_second_cubed != None:
            return self.__pounds_per_second_cubed
        self.__pounds_per_second_cubed = self.__convert_from_base(HeatFluxUnits.PoundPerSecondCubed)
        return self.__pounds_per_second_cubed

    
    @property
    def nanowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_square_meter != None:
            return self.__nanowatts_per_square_meter
        self.__nanowatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.NanowattPerSquareMeter)
        return self.__nanowatts_per_square_meter

    
    @property
    def microwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__microwatts_per_square_meter != None:
            return self.__microwatts_per_square_meter
        self.__microwatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.MicrowattPerSquareMeter)
        return self.__microwatts_per_square_meter

    
    @property
    def milliwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_square_meter != None:
            return self.__milliwatts_per_square_meter
        self.__milliwatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.MilliwattPerSquareMeter)
        return self.__milliwatts_per_square_meter

    
    @property
    def centiwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__centiwatts_per_square_meter != None:
            return self.__centiwatts_per_square_meter
        self.__centiwatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.CentiwattPerSquareMeter)
        return self.__centiwatts_per_square_meter

    
    @property
    def deciwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__deciwatts_per_square_meter != None:
            return self.__deciwatts_per_square_meter
        self.__deciwatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.DeciwattPerSquareMeter)
        return self.__deciwatts_per_square_meter

    
    @property
    def kilowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_square_meter != None:
            return self.__kilowatts_per_square_meter
        self.__kilowatts_per_square_meter = self.__convert_from_base(HeatFluxUnits.KilowattPerSquareMeter)
        return self.__kilowatts_per_square_meter

    
    @property
    def kilocalories_per_second_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilocalories_per_second_square_centimeter != None:
            return self.__kilocalories_per_second_square_centimeter
        self.__kilocalories_per_second_square_centimeter = self.__convert_from_base(HeatFluxUnits.KilocaloriePerSecondSquareCentimeter)
        return self.__kilocalories_per_second_square_centimeter

    
    def to_string(self, unit: HeatFluxUnits = HeatFluxUnits.WattPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the HeatFlux to a string.
        
        Note: the default format for HeatFlux is WattPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the HeatFlux. Default is 'WattPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == HeatFluxUnits.WattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.watts_per_square_meter, fractional_digits)} W/m²"""
        
        if unit == HeatFluxUnits.WattPerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.watts_per_square_inch, fractional_digits)} W/in²"""
        
        if unit == HeatFluxUnits.WattPerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.watts_per_square_foot, fractional_digits)} W/ft²"""
        
        if unit == HeatFluxUnits.BtuPerSecondSquareInch:
            return f"""{super()._truncate_fraction_digits(self.btus_per_second_square_inch, fractional_digits)} BTU/s·in²"""
        
        if unit == HeatFluxUnits.BtuPerSecondSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.btus_per_second_square_foot, fractional_digits)} BTU/s·ft²"""
        
        if unit == HeatFluxUnits.BtuPerMinuteSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.btus_per_minute_square_foot, fractional_digits)} BTU/min·ft²"""
        
        if unit == HeatFluxUnits.BtuPerHourSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.btus_per_hour_square_foot, fractional_digits)} BTU/h·ft²"""
        
        if unit == HeatFluxUnits.CaloriePerSecondSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.calories_per_second_square_centimeter, fractional_digits)} cal/s·cm²"""
        
        if unit == HeatFluxUnits.KilocaloriePerHourSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_hour_square_meter, fractional_digits)} kcal/h·m²"""
        
        if unit == HeatFluxUnits.PoundForcePerFootSecond:
            return f"""{super()._truncate_fraction_digits(self.pounds_force_per_foot_second, fractional_digits)} lbf/(ft·s)"""
        
        if unit == HeatFluxUnits.PoundPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_second_cubed, fractional_digits)} lb/s³"""
        
        if unit == HeatFluxUnits.NanowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.nanowatts_per_square_meter, fractional_digits)} nW/m²"""
        
        if unit == HeatFluxUnits.MicrowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.microwatts_per_square_meter, fractional_digits)} μW/m²"""
        
        if unit == HeatFluxUnits.MilliwattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.milliwatts_per_square_meter, fractional_digits)} mW/m²"""
        
        if unit == HeatFluxUnits.CentiwattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.centiwatts_per_square_meter, fractional_digits)} cW/m²"""
        
        if unit == HeatFluxUnits.DeciwattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.deciwatts_per_square_meter, fractional_digits)} dW/m²"""
        
        if unit == HeatFluxUnits.KilowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilowatts_per_square_meter, fractional_digits)} kW/m²"""
        
        if unit == HeatFluxUnits.KilocaloriePerSecondSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_second_square_centimeter, fractional_digits)} kcal/s·cm²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: HeatFluxUnits = HeatFluxUnits.WattPerSquareMeter) -> str:
        """
        Get HeatFlux unit abbreviation.
        Note! the default abbreviation for HeatFlux is WattPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == HeatFluxUnits.WattPerSquareMeter:
            return """W/m²"""
        
        if unit_abbreviation == HeatFluxUnits.WattPerSquareInch:
            return """W/in²"""
        
        if unit_abbreviation == HeatFluxUnits.WattPerSquareFoot:
            return """W/ft²"""
        
        if unit_abbreviation == HeatFluxUnits.BtuPerSecondSquareInch:
            return """BTU/s·in²"""
        
        if unit_abbreviation == HeatFluxUnits.BtuPerSecondSquareFoot:
            return """BTU/s·ft²"""
        
        if unit_abbreviation == HeatFluxUnits.BtuPerMinuteSquareFoot:
            return """BTU/min·ft²"""
        
        if unit_abbreviation == HeatFluxUnits.BtuPerHourSquareFoot:
            return """BTU/h·ft²"""
        
        if unit_abbreviation == HeatFluxUnits.CaloriePerSecondSquareCentimeter:
            return """cal/s·cm²"""
        
        if unit_abbreviation == HeatFluxUnits.KilocaloriePerHourSquareMeter:
            return """kcal/h·m²"""
        
        if unit_abbreviation == HeatFluxUnits.PoundForcePerFootSecond:
            return """lbf/(ft·s)"""
        
        if unit_abbreviation == HeatFluxUnits.PoundPerSecondCubed:
            return """lb/s³"""
        
        if unit_abbreviation == HeatFluxUnits.NanowattPerSquareMeter:
            return """nW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.MicrowattPerSquareMeter:
            return """μW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.MilliwattPerSquareMeter:
            return """mW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.CentiwattPerSquareMeter:
            return """cW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.DeciwattPerSquareMeter:
            return """dW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.KilowattPerSquareMeter:
            return """kW/m²"""
        
        if unit_abbreviation == HeatFluxUnits.KilocaloriePerSecondSquareCentimeter:
            return """kcal/s·cm²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class SpecificFuelConsumptionUnits(Enum):
        """
            SpecificFuelConsumptionUnits enumeration
        """
        
        PoundMassPerPoundForceHour = 'PoundMassPerPoundForceHour'
        """
            
        """
        
        KilogramPerKilogramForceHour = 'KilogramPerKilogramForceHour'
        """
            
        """
        
        GramPerKiloNewtonSecond = 'GramPerKiloNewtonSecond'
        """
            
        """
        
        KilogramPerKiloNewtonSecond = 'KilogramPerKiloNewtonSecond'
        """
            
        """
        

class SpecificFuelConsumptionDto:
    """
    A DTO representation of a SpecificFuelConsumption

    Attributes:
        value (float): The value of the SpecificFuelConsumption.
        unit (SpecificFuelConsumptionUnits): The specific unit that the SpecificFuelConsumption value is representing.
    """

    def __init__(self, value: float, unit: SpecificFuelConsumptionUnits):
        """
        Create a new DTO representation of a SpecificFuelConsumption

        Parameters:
            value (float): The value of the SpecificFuelConsumption.
            unit (SpecificFuelConsumptionUnits): The specific unit that the SpecificFuelConsumption value is representing.
        """
        self.value: float = value
        """
        The value of the SpecificFuelConsumption
        """
        self.unit: SpecificFuelConsumptionUnits = unit
        """
        The specific unit that the SpecificFuelConsumption value is representing
        """

    def to_json(self):
        """
        Get a SpecificFuelConsumption DTO JSON object representing the current unit.

        :return: JSON object represents SpecificFuelConsumption DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "GramPerKiloNewtonSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of SpecificFuelConsumption DTO from a json representation.

        :param data: The SpecificFuelConsumption DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "GramPerKiloNewtonSecond"}
        :return: A new instance of SpecificFuelConsumptionDto.
        :rtype: SpecificFuelConsumptionDto
        """
        return SpecificFuelConsumptionDto(value=data["value"], unit=SpecificFuelConsumptionUnits(data["unit"]))


class SpecificFuelConsumption(AbstractMeasure):
    """
    SFC is the fuel efficiency of an engine design with respect to thrust output

    Args:
        value (float): The value.
        from_unit (SpecificFuelConsumptionUnits): The SpecificFuelConsumption unit to create from, The default unit is GramPerKiloNewtonSecond
    """
    def __init__(self, value: float, from_unit: SpecificFuelConsumptionUnits = SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__pounds_mass_per_pound_force_hour = None
        
        self.__kilograms_per_kilogram_force_hour = None
        
        self.__grams_per_kilo_newton_second = None
        
        self.__kilograms_per_kilo_newton_second = None
        

    def convert(self, unit: SpecificFuelConsumptionUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: SpecificFuelConsumptionUnits = SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond) -> SpecificFuelConsumptionDto:
        """
        Get a new instance of SpecificFuelConsumption DTO representing the current unit.

        :param hold_in_unit: The specific SpecificFuelConsumption unit to store the SpecificFuelConsumption value in the DTO representation.
        :type hold_in_unit: SpecificFuelConsumptionUnits
        :return: A new instance of SpecificFuelConsumptionDto.
        :rtype: SpecificFuelConsumptionDto
        """
        return SpecificFuelConsumptionDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: SpecificFuelConsumptionUnits = SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond):
        """
        Get a SpecificFuelConsumption DTO JSON object representing the current unit.

        :param hold_in_unit: The specific SpecificFuelConsumption unit to store the SpecificFuelConsumption value in the DTO representation.
        :type hold_in_unit: SpecificFuelConsumptionUnits
        :return: JSON object represents SpecificFuelConsumption DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "GramPerKiloNewtonSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(specific_fuel_consumption_dto: SpecificFuelConsumptionDto):
        """
        Obtain a new instance of SpecificFuelConsumption from a DTO unit object.

        :param specific_fuel_consumption_dto: The SpecificFuelConsumption DTO representation.
        :type specific_fuel_consumption_dto: SpecificFuelConsumptionDto
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption(specific_fuel_consumption_dto.value, specific_fuel_consumption_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of SpecificFuelConsumption from a DTO unit json representation.

        :param data: The SpecificFuelConsumption DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "GramPerKiloNewtonSecond"}
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption.from_dto(SpecificFuelConsumptionDto.from_json(data))

    def __convert_from_base(self, from_unit: SpecificFuelConsumptionUnits) -> float:
        value = self._value
        
        if from_unit == SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour:
            return (value / 28.33)
        
        if from_unit == SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour:
            return (value / 28.33)
        
        if from_unit == SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond:
            return (value)
        
        if from_unit == SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond:
            return ((value) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpecificFuelConsumptionUnits) -> float:
        
        if to_unit == SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour:
            return (value * 28.33)
        
        if to_unit == SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour:
            return (value * 28.33)
        
        if to_unit == SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond:
            return (value)
        
        if to_unit == SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond:
            return ((value) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_pounds_mass_per_pound_force_hour(pounds_mass_per_pound_force_hour: float):
        """
        Create a new instance of SpecificFuelConsumption from a value in pounds_mass_per_pound_force_hour.

        

        :param meters: The SpecificFuelConsumption value in pounds_mass_per_pound_force_hour.
        :type pounds_mass_per_pound_force_hour: float
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption(pounds_mass_per_pound_force_hour, SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour)

    
    @staticmethod
    def from_kilograms_per_kilogram_force_hour(kilograms_per_kilogram_force_hour: float):
        """
        Create a new instance of SpecificFuelConsumption from a value in kilograms_per_kilogram_force_hour.

        

        :param meters: The SpecificFuelConsumption value in kilograms_per_kilogram_force_hour.
        :type kilograms_per_kilogram_force_hour: float
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption(kilograms_per_kilogram_force_hour, SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour)

    
    @staticmethod
    def from_grams_per_kilo_newton_second(grams_per_kilo_newton_second: float):
        """
        Create a new instance of SpecificFuelConsumption from a value in grams_per_kilo_newton_second.

        

        :param meters: The SpecificFuelConsumption value in grams_per_kilo_newton_second.
        :type grams_per_kilo_newton_second: float
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption(grams_per_kilo_newton_second, SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond)

    
    @staticmethod
    def from_kilograms_per_kilo_newton_second(kilograms_per_kilo_newton_second: float):
        """
        Create a new instance of SpecificFuelConsumption from a value in kilograms_per_kilo_newton_second.

        

        :param meters: The SpecificFuelConsumption value in kilograms_per_kilo_newton_second.
        :type kilograms_per_kilo_newton_second: float
        :return: A new instance of SpecificFuelConsumption.
        :rtype: SpecificFuelConsumption
        """
        return SpecificFuelConsumption(kilograms_per_kilo_newton_second, SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond)

    
    @property
    def pounds_mass_per_pound_force_hour(self) -> float:
        """
        
        """
        if self.__pounds_mass_per_pound_force_hour != None:
            return self.__pounds_mass_per_pound_force_hour
        self.__pounds_mass_per_pound_force_hour = self.__convert_from_base(SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour)
        return self.__pounds_mass_per_pound_force_hour

    
    @property
    def kilograms_per_kilogram_force_hour(self) -> float:
        """
        
        """
        if self.__kilograms_per_kilogram_force_hour != None:
            return self.__kilograms_per_kilogram_force_hour
        self.__kilograms_per_kilogram_force_hour = self.__convert_from_base(SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour)
        return self.__kilograms_per_kilogram_force_hour

    
    @property
    def grams_per_kilo_newton_second(self) -> float:
        """
        
        """
        if self.__grams_per_kilo_newton_second != None:
            return self.__grams_per_kilo_newton_second
        self.__grams_per_kilo_newton_second = self.__convert_from_base(SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond)
        return self.__grams_per_kilo_newton_second

    
    @property
    def kilograms_per_kilo_newton_second(self) -> float:
        """
        
        """
        if self.__kilograms_per_kilo_newton_second != None:
            return self.__kilograms_per_kilo_newton_second
        self.__kilograms_per_kilo_newton_second = self.__convert_from_base(SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond)
        return self.__kilograms_per_kilo_newton_second

    
    def to_string(self, unit: SpecificFuelConsumptionUnits = SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond, fractional_digits: int = None) -> str:
        """
        Format the SpecificFuelConsumption to a string.
        
        Note: the default format for SpecificFuelConsumption is GramPerKiloNewtonSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the SpecificFuelConsumption. Default is 'GramPerKiloNewtonSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour:
            return f"""{super()._truncate_fraction_digits(self.pounds_mass_per_pound_force_hour, fractional_digits)} lb/(lbf·h)"""
        
        if unit == SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_kilogram_force_hour, fractional_digits)} kg/(kgf�h)"""
        
        if unit == SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.grams_per_kilo_newton_second, fractional_digits)} g/(kN�s)"""
        
        if unit == SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_kilo_newton_second, fractional_digits)} kg/(kN�s)"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpecificFuelConsumptionUnits = SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond) -> str:
        """
        Get SpecificFuelConsumption unit abbreviation.
        Note! the default abbreviation for SpecificFuelConsumption is GramPerKiloNewtonSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpecificFuelConsumptionUnits.PoundMassPerPoundForceHour:
            return """lb/(lbf·h)"""
        
        if unit_abbreviation == SpecificFuelConsumptionUnits.KilogramPerKilogramForceHour:
            return """kg/(kgf�h)"""
        
        if unit_abbreviation == SpecificFuelConsumptionUnits.GramPerKiloNewtonSecond:
            return """g/(kN�s)"""
        
        if unit_abbreviation == SpecificFuelConsumptionUnits.KilogramPerKiloNewtonSecond:
            return """kg/(kN�s)"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class SpecificEnergyUnits(Enum):
        """
            SpecificEnergyUnits enumeration
        """
        
        JoulePerKilogram = 'JoulePerKilogram'
        """
            
        """
        
        MegaJoulePerTonne = 'MegaJoulePerTonne'
        """
            
        """
        
        CaloriePerGram = 'CaloriePerGram'
        """
            
        """
        
        WattHourPerKilogram = 'WattHourPerKilogram'
        """
            
        """
        
        WattDayPerKilogram = 'WattDayPerKilogram'
        """
            
        """
        
        WattDayPerTonne = 'WattDayPerTonne'
        """
            
        """
        
        WattDayPerShortTon = 'WattDayPerShortTon'
        """
            
        """
        
        WattHourPerPound = 'WattHourPerPound'
        """
            
        """
        
        BtuPerPound = 'BtuPerPound'
        """
            
        """
        
        KilojoulePerKilogram = 'KilojoulePerKilogram'
        """
            
        """
        
        MegajoulePerKilogram = 'MegajoulePerKilogram'
        """
            
        """
        
        KilocaloriePerGram = 'KilocaloriePerGram'
        """
            
        """
        
        KilowattHourPerKilogram = 'KilowattHourPerKilogram'
        """
            
        """
        
        MegawattHourPerKilogram = 'MegawattHourPerKilogram'
        """
            
        """
        
        GigawattHourPerKilogram = 'GigawattHourPerKilogram'
        """
            
        """
        
        KilowattDayPerKilogram = 'KilowattDayPerKilogram'
        """
            
        """
        
        MegawattDayPerKilogram = 'MegawattDayPerKilogram'
        """
            
        """
        
        GigawattDayPerKilogram = 'GigawattDayPerKilogram'
        """
            
        """
        
        TerawattDayPerKilogram = 'TerawattDayPerKilogram'
        """
            
        """
        
        KilowattDayPerTonne = 'KilowattDayPerTonne'
        """
            
        """
        
        MegawattDayPerTonne = 'MegawattDayPerTonne'
        """
            
        """
        
        GigawattDayPerTonne = 'GigawattDayPerTonne'
        """
            
        """
        
        TerawattDayPerTonne = 'TerawattDayPerTonne'
        """
            
        """
        
        KilowattDayPerShortTon = 'KilowattDayPerShortTon'
        """
            
        """
        
        MegawattDayPerShortTon = 'MegawattDayPerShortTon'
        """
            
        """
        
        GigawattDayPerShortTon = 'GigawattDayPerShortTon'
        """
            
        """
        
        TerawattDayPerShortTon = 'TerawattDayPerShortTon'
        """
            
        """
        
        KilowattHourPerPound = 'KilowattHourPerPound'
        """
            
        """
        
        MegawattHourPerPound = 'MegawattHourPerPound'
        """
            
        """
        
        GigawattHourPerPound = 'GigawattHourPerPound'
        """
            
        """
        

class SpecificEnergyDto:
    """
    A DTO representation of a SpecificEnergy

    Attributes:
        value (float): The value of the SpecificEnergy.
        unit (SpecificEnergyUnits): The specific unit that the SpecificEnergy value is representing.
    """

    def __init__(self, value: float, unit: SpecificEnergyUnits):
        """
        Create a new DTO representation of a SpecificEnergy

        Parameters:
            value (float): The value of the SpecificEnergy.
            unit (SpecificEnergyUnits): The specific unit that the SpecificEnergy value is representing.
        """
        self.value: float = value
        """
        The value of the SpecificEnergy
        """
        self.unit: SpecificEnergyUnits = unit
        """
        The specific unit that the SpecificEnergy value is representing
        """

    def to_json(self):
        """
        Get a SpecificEnergy DTO JSON object representing the current unit.

        :return: JSON object represents SpecificEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKilogram"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of SpecificEnergy DTO from a json representation.

        :param data: The SpecificEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKilogram"}
        :return: A new instance of SpecificEnergyDto.
        :rtype: SpecificEnergyDto
        """
        return SpecificEnergyDto(value=data["value"], unit=SpecificEnergyUnits(data["unit"]))


class SpecificEnergy(AbstractMeasure):
    """
    The SpecificEnergy

    Args:
        value (float): The value.
        from_unit (SpecificEnergyUnits): The SpecificEnergy unit to create from, The default unit is JoulePerKilogram
    """
    def __init__(self, value: float, from_unit: SpecificEnergyUnits = SpecificEnergyUnits.JoulePerKilogram):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_kilogram = None
        
        self.__mega_joules_per_tonne = None
        
        self.__calories_per_gram = None
        
        self.__watt_hours_per_kilogram = None
        
        self.__watt_days_per_kilogram = None
        
        self.__watt_days_per_tonne = None
        
        self.__watt_days_per_short_ton = None
        
        self.__watt_hours_per_pound = None
        
        self.__btu_per_pound = None
        
        self.__kilojoules_per_kilogram = None
        
        self.__megajoules_per_kilogram = None
        
        self.__kilocalories_per_gram = None
        
        self.__kilowatt_hours_per_kilogram = None
        
        self.__megawatt_hours_per_kilogram = None
        
        self.__gigawatt_hours_per_kilogram = None
        
        self.__kilowatt_days_per_kilogram = None
        
        self.__megawatt_days_per_kilogram = None
        
        self.__gigawatt_days_per_kilogram = None
        
        self.__terawatt_days_per_kilogram = None
        
        self.__kilowatt_days_per_tonne = None
        
        self.__megawatt_days_per_tonne = None
        
        self.__gigawatt_days_per_tonne = None
        
        self.__terawatt_days_per_tonne = None
        
        self.__kilowatt_days_per_short_ton = None
        
        self.__megawatt_days_per_short_ton = None
        
        self.__gigawatt_days_per_short_ton = None
        
        self.__terawatt_days_per_short_ton = None
        
        self.__kilowatt_hours_per_pound = None
        
        self.__megawatt_hours_per_pound = None
        
        self.__gigawatt_hours_per_pound = None
        

    def convert(self, unit: SpecificEnergyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: SpecificEnergyUnits = SpecificEnergyUnits.JoulePerKilogram) -> SpecificEnergyDto:
        """
        Get a new instance of SpecificEnergy DTO representing the current unit.

        :param hold_in_unit: The specific SpecificEnergy unit to store the SpecificEnergy value in the DTO representation.
        :type hold_in_unit: SpecificEnergyUnits
        :return: A new instance of SpecificEnergyDto.
        :rtype: SpecificEnergyDto
        """
        return SpecificEnergyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: SpecificEnergyUnits = SpecificEnergyUnits.JoulePerKilogram):
        """
        Get a SpecificEnergy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific SpecificEnergy unit to store the SpecificEnergy value in the DTO representation.
        :type hold_in_unit: SpecificEnergyUnits
        :return: JSON object represents SpecificEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKilogram"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(specific_energy_dto: SpecificEnergyDto):
        """
        Obtain a new instance of SpecificEnergy from a DTO unit object.

        :param specific_energy_dto: The SpecificEnergy DTO representation.
        :type specific_energy_dto: SpecificEnergyDto
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(specific_energy_dto.value, specific_energy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of SpecificEnergy from a DTO unit json representation.

        :param data: The SpecificEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKilogram"}
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy.from_dto(SpecificEnergyDto.from_json(data))

    def __convert_from_base(self, from_unit: SpecificEnergyUnits) -> float:
        value = self._value
        
        if from_unit == SpecificEnergyUnits.JoulePerKilogram:
            return (value)
        
        if from_unit == SpecificEnergyUnits.MegaJoulePerTonne:
            return (value / 1e3)
        
        if from_unit == SpecificEnergyUnits.CaloriePerGram:
            return (value / 4.184e3)
        
        if from_unit == SpecificEnergyUnits.WattHourPerKilogram:
            return (value / 3.6e3)
        
        if from_unit == SpecificEnergyUnits.WattDayPerKilogram:
            return (value / (24 * 3.6e3))
        
        if from_unit == SpecificEnergyUnits.WattDayPerTonne:
            return (value / ((24 * 3.6e3) / 1e3))
        
        if from_unit == SpecificEnergyUnits.WattDayPerShortTon:
            return (value / ((24 * 3.6e3) / 9.0718474e2))
        
        if from_unit == SpecificEnergyUnits.WattHourPerPound:
            return (value / 7.93664e3)
        
        if from_unit == SpecificEnergyUnits.BtuPerPound:
            return (value / 2326.000075362)
        
        if from_unit == SpecificEnergyUnits.KilojoulePerKilogram:
            return ((value) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegajoulePerKilogram:
            return ((value) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.KilocaloriePerGram:
            return ((value / 4.184e3) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.KilowattHourPerKilogram:
            return ((value / 3.6e3) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegawattHourPerKilogram:
            return ((value / 3.6e3) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.GigawattHourPerKilogram:
            return ((value / 3.6e3) / 1000000000.0)
        
        if from_unit == SpecificEnergyUnits.KilowattDayPerKilogram:
            return ((value / (24 * 3.6e3)) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegawattDayPerKilogram:
            return ((value / (24 * 3.6e3)) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.GigawattDayPerKilogram:
            return ((value / (24 * 3.6e3)) / 1000000000.0)
        
        if from_unit == SpecificEnergyUnits.TerawattDayPerKilogram:
            return ((value / (24 * 3.6e3)) / 1000000000000.0)
        
        if from_unit == SpecificEnergyUnits.KilowattDayPerTonne:
            return ((value / ((24 * 3.6e3) / 1e3)) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegawattDayPerTonne:
            return ((value / ((24 * 3.6e3) / 1e3)) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.GigawattDayPerTonne:
            return ((value / ((24 * 3.6e3) / 1e3)) / 1000000000.0)
        
        if from_unit == SpecificEnergyUnits.TerawattDayPerTonne:
            return ((value / ((24 * 3.6e3) / 1e3)) / 1000000000000.0)
        
        if from_unit == SpecificEnergyUnits.KilowattDayPerShortTon:
            return ((value / ((24 * 3.6e3) / 9.0718474e2)) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegawattDayPerShortTon:
            return ((value / ((24 * 3.6e3) / 9.0718474e2)) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.GigawattDayPerShortTon:
            return ((value / ((24 * 3.6e3) / 9.0718474e2)) / 1000000000.0)
        
        if from_unit == SpecificEnergyUnits.TerawattDayPerShortTon:
            return ((value / ((24 * 3.6e3) / 9.0718474e2)) / 1000000000000.0)
        
        if from_unit == SpecificEnergyUnits.KilowattHourPerPound:
            return ((value / 7.93664e3) / 1000.0)
        
        if from_unit == SpecificEnergyUnits.MegawattHourPerPound:
            return ((value / 7.93664e3) / 1000000.0)
        
        if from_unit == SpecificEnergyUnits.GigawattHourPerPound:
            return ((value / 7.93664e3) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpecificEnergyUnits) -> float:
        
        if to_unit == SpecificEnergyUnits.JoulePerKilogram:
            return (value)
        
        if to_unit == SpecificEnergyUnits.MegaJoulePerTonne:
            return (value * 1e3)
        
        if to_unit == SpecificEnergyUnits.CaloriePerGram:
            return (value * 4.184e3)
        
        if to_unit == SpecificEnergyUnits.WattHourPerKilogram:
            return (value * 3.6e3)
        
        if to_unit == SpecificEnergyUnits.WattDayPerKilogram:
            return (value * (24 * 3.6e3))
        
        if to_unit == SpecificEnergyUnits.WattDayPerTonne:
            return (value * ((24 * 3.6e3) / 1e3))
        
        if to_unit == SpecificEnergyUnits.WattDayPerShortTon:
            return (value * ((24 * 3.6e3) / 9.0718474e2))
        
        if to_unit == SpecificEnergyUnits.WattHourPerPound:
            return (value * 7.93664e3)
        
        if to_unit == SpecificEnergyUnits.BtuPerPound:
            return (value * 2326.000075362)
        
        if to_unit == SpecificEnergyUnits.KilojoulePerKilogram:
            return ((value) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegajoulePerKilogram:
            return ((value) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.KilocaloriePerGram:
            return ((value * 4.184e3) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.KilowattHourPerKilogram:
            return ((value * 3.6e3) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegawattHourPerKilogram:
            return ((value * 3.6e3) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.GigawattHourPerKilogram:
            return ((value * 3.6e3) * 1000000000.0)
        
        if to_unit == SpecificEnergyUnits.KilowattDayPerKilogram:
            return ((value * (24 * 3.6e3)) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegawattDayPerKilogram:
            return ((value * (24 * 3.6e3)) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.GigawattDayPerKilogram:
            return ((value * (24 * 3.6e3)) * 1000000000.0)
        
        if to_unit == SpecificEnergyUnits.TerawattDayPerKilogram:
            return ((value * (24 * 3.6e3)) * 1000000000000.0)
        
        if to_unit == SpecificEnergyUnits.KilowattDayPerTonne:
            return ((value * ((24 * 3.6e3) / 1e3)) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegawattDayPerTonne:
            return ((value * ((24 * 3.6e3) / 1e3)) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.GigawattDayPerTonne:
            return ((value * ((24 * 3.6e3) / 1e3)) * 1000000000.0)
        
        if to_unit == SpecificEnergyUnits.TerawattDayPerTonne:
            return ((value * ((24 * 3.6e3) / 1e3)) * 1000000000000.0)
        
        if to_unit == SpecificEnergyUnits.KilowattDayPerShortTon:
            return ((value * ((24 * 3.6e3) / 9.0718474e2)) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegawattDayPerShortTon:
            return ((value * ((24 * 3.6e3) / 9.0718474e2)) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.GigawattDayPerShortTon:
            return ((value * ((24 * 3.6e3) / 9.0718474e2)) * 1000000000.0)
        
        if to_unit == SpecificEnergyUnits.TerawattDayPerShortTon:
            return ((value * ((24 * 3.6e3) / 9.0718474e2)) * 1000000000000.0)
        
        if to_unit == SpecificEnergyUnits.KilowattHourPerPound:
            return ((value * 7.93664e3) * 1000.0)
        
        if to_unit == SpecificEnergyUnits.MegawattHourPerPound:
            return ((value * 7.93664e3) * 1000000.0)
        
        if to_unit == SpecificEnergyUnits.GigawattHourPerPound:
            return ((value * 7.93664e3) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_kilogram(joules_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in joules_per_kilogram.

        

        :param meters: The SpecificEnergy value in joules_per_kilogram.
        :type joules_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(joules_per_kilogram, SpecificEnergyUnits.JoulePerKilogram)

    
    @staticmethod
    def from_mega_joules_per_tonne(mega_joules_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in mega_joules_per_tonne.

        

        :param meters: The SpecificEnergy value in mega_joules_per_tonne.
        :type mega_joules_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(mega_joules_per_tonne, SpecificEnergyUnits.MegaJoulePerTonne)

    
    @staticmethod
    def from_calories_per_gram(calories_per_gram: float):
        """
        Create a new instance of SpecificEnergy from a value in calories_per_gram.

        

        :param meters: The SpecificEnergy value in calories_per_gram.
        :type calories_per_gram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(calories_per_gram, SpecificEnergyUnits.CaloriePerGram)

    
    @staticmethod
    def from_watt_hours_per_kilogram(watt_hours_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in watt_hours_per_kilogram.

        

        :param meters: The SpecificEnergy value in watt_hours_per_kilogram.
        :type watt_hours_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(watt_hours_per_kilogram, SpecificEnergyUnits.WattHourPerKilogram)

    
    @staticmethod
    def from_watt_days_per_kilogram(watt_days_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in watt_days_per_kilogram.

        

        :param meters: The SpecificEnergy value in watt_days_per_kilogram.
        :type watt_days_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(watt_days_per_kilogram, SpecificEnergyUnits.WattDayPerKilogram)

    
    @staticmethod
    def from_watt_days_per_tonne(watt_days_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in watt_days_per_tonne.

        

        :param meters: The SpecificEnergy value in watt_days_per_tonne.
        :type watt_days_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(watt_days_per_tonne, SpecificEnergyUnits.WattDayPerTonne)

    
    @staticmethod
    def from_watt_days_per_short_ton(watt_days_per_short_ton: float):
        """
        Create a new instance of SpecificEnergy from a value in watt_days_per_short_ton.

        

        :param meters: The SpecificEnergy value in watt_days_per_short_ton.
        :type watt_days_per_short_ton: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(watt_days_per_short_ton, SpecificEnergyUnits.WattDayPerShortTon)

    
    @staticmethod
    def from_watt_hours_per_pound(watt_hours_per_pound: float):
        """
        Create a new instance of SpecificEnergy from a value in watt_hours_per_pound.

        

        :param meters: The SpecificEnergy value in watt_hours_per_pound.
        :type watt_hours_per_pound: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(watt_hours_per_pound, SpecificEnergyUnits.WattHourPerPound)

    
    @staticmethod
    def from_btu_per_pound(btu_per_pound: float):
        """
        Create a new instance of SpecificEnergy from a value in btu_per_pound.

        

        :param meters: The SpecificEnergy value in btu_per_pound.
        :type btu_per_pound: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(btu_per_pound, SpecificEnergyUnits.BtuPerPound)

    
    @staticmethod
    def from_kilojoules_per_kilogram(kilojoules_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in kilojoules_per_kilogram.

        

        :param meters: The SpecificEnergy value in kilojoules_per_kilogram.
        :type kilojoules_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilojoules_per_kilogram, SpecificEnergyUnits.KilojoulePerKilogram)

    
    @staticmethod
    def from_megajoules_per_kilogram(megajoules_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in megajoules_per_kilogram.

        

        :param meters: The SpecificEnergy value in megajoules_per_kilogram.
        :type megajoules_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megajoules_per_kilogram, SpecificEnergyUnits.MegajoulePerKilogram)

    
    @staticmethod
    def from_kilocalories_per_gram(kilocalories_per_gram: float):
        """
        Create a new instance of SpecificEnergy from a value in kilocalories_per_gram.

        

        :param meters: The SpecificEnergy value in kilocalories_per_gram.
        :type kilocalories_per_gram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilocalories_per_gram, SpecificEnergyUnits.KilocaloriePerGram)

    
    @staticmethod
    def from_kilowatt_hours_per_kilogram(kilowatt_hours_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in kilowatt_hours_per_kilogram.

        

        :param meters: The SpecificEnergy value in kilowatt_hours_per_kilogram.
        :type kilowatt_hours_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilowatt_hours_per_kilogram, SpecificEnergyUnits.KilowattHourPerKilogram)

    
    @staticmethod
    def from_megawatt_hours_per_kilogram(megawatt_hours_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in megawatt_hours_per_kilogram.

        

        :param meters: The SpecificEnergy value in megawatt_hours_per_kilogram.
        :type megawatt_hours_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megawatt_hours_per_kilogram, SpecificEnergyUnits.MegawattHourPerKilogram)

    
    @staticmethod
    def from_gigawatt_hours_per_kilogram(gigawatt_hours_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in gigawatt_hours_per_kilogram.

        

        :param meters: The SpecificEnergy value in gigawatt_hours_per_kilogram.
        :type gigawatt_hours_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(gigawatt_hours_per_kilogram, SpecificEnergyUnits.GigawattHourPerKilogram)

    
    @staticmethod
    def from_kilowatt_days_per_kilogram(kilowatt_days_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in kilowatt_days_per_kilogram.

        

        :param meters: The SpecificEnergy value in kilowatt_days_per_kilogram.
        :type kilowatt_days_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilowatt_days_per_kilogram, SpecificEnergyUnits.KilowattDayPerKilogram)

    
    @staticmethod
    def from_megawatt_days_per_kilogram(megawatt_days_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in megawatt_days_per_kilogram.

        

        :param meters: The SpecificEnergy value in megawatt_days_per_kilogram.
        :type megawatt_days_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megawatt_days_per_kilogram, SpecificEnergyUnits.MegawattDayPerKilogram)

    
    @staticmethod
    def from_gigawatt_days_per_kilogram(gigawatt_days_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in gigawatt_days_per_kilogram.

        

        :param meters: The SpecificEnergy value in gigawatt_days_per_kilogram.
        :type gigawatt_days_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(gigawatt_days_per_kilogram, SpecificEnergyUnits.GigawattDayPerKilogram)

    
    @staticmethod
    def from_terawatt_days_per_kilogram(terawatt_days_per_kilogram: float):
        """
        Create a new instance of SpecificEnergy from a value in terawatt_days_per_kilogram.

        

        :param meters: The SpecificEnergy value in terawatt_days_per_kilogram.
        :type terawatt_days_per_kilogram: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(terawatt_days_per_kilogram, SpecificEnergyUnits.TerawattDayPerKilogram)

    
    @staticmethod
    def from_kilowatt_days_per_tonne(kilowatt_days_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in kilowatt_days_per_tonne.

        

        :param meters: The SpecificEnergy value in kilowatt_days_per_tonne.
        :type kilowatt_days_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilowatt_days_per_tonne, SpecificEnergyUnits.KilowattDayPerTonne)

    
    @staticmethod
    def from_megawatt_days_per_tonne(megawatt_days_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in megawatt_days_per_tonne.

        

        :param meters: The SpecificEnergy value in megawatt_days_per_tonne.
        :type megawatt_days_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megawatt_days_per_tonne, SpecificEnergyUnits.MegawattDayPerTonne)

    
    @staticmethod
    def from_gigawatt_days_per_tonne(gigawatt_days_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in gigawatt_days_per_tonne.

        

        :param meters: The SpecificEnergy value in gigawatt_days_per_tonne.
        :type gigawatt_days_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(gigawatt_days_per_tonne, SpecificEnergyUnits.GigawattDayPerTonne)

    
    @staticmethod
    def from_terawatt_days_per_tonne(terawatt_days_per_tonne: float):
        """
        Create a new instance of SpecificEnergy from a value in terawatt_days_per_tonne.

        

        :param meters: The SpecificEnergy value in terawatt_days_per_tonne.
        :type terawatt_days_per_tonne: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(terawatt_days_per_tonne, SpecificEnergyUnits.TerawattDayPerTonne)

    
    @staticmethod
    def from_kilowatt_days_per_short_ton(kilowatt_days_per_short_ton: float):
        """
        Create a new instance of SpecificEnergy from a value in kilowatt_days_per_short_ton.

        

        :param meters: The SpecificEnergy value in kilowatt_days_per_short_ton.
        :type kilowatt_days_per_short_ton: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilowatt_days_per_short_ton, SpecificEnergyUnits.KilowattDayPerShortTon)

    
    @staticmethod
    def from_megawatt_days_per_short_ton(megawatt_days_per_short_ton: float):
        """
        Create a new instance of SpecificEnergy from a value in megawatt_days_per_short_ton.

        

        :param meters: The SpecificEnergy value in megawatt_days_per_short_ton.
        :type megawatt_days_per_short_ton: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megawatt_days_per_short_ton, SpecificEnergyUnits.MegawattDayPerShortTon)

    
    @staticmethod
    def from_gigawatt_days_per_short_ton(gigawatt_days_per_short_ton: float):
        """
        Create a new instance of SpecificEnergy from a value in gigawatt_days_per_short_ton.

        

        :param meters: The SpecificEnergy value in gigawatt_days_per_short_ton.
        :type gigawatt_days_per_short_ton: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(gigawatt_days_per_short_ton, SpecificEnergyUnits.GigawattDayPerShortTon)

    
    @staticmethod
    def from_terawatt_days_per_short_ton(terawatt_days_per_short_ton: float):
        """
        Create a new instance of SpecificEnergy from a value in terawatt_days_per_short_ton.

        

        :param meters: The SpecificEnergy value in terawatt_days_per_short_ton.
        :type terawatt_days_per_short_ton: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(terawatt_days_per_short_ton, SpecificEnergyUnits.TerawattDayPerShortTon)

    
    @staticmethod
    def from_kilowatt_hours_per_pound(kilowatt_hours_per_pound: float):
        """
        Create a new instance of SpecificEnergy from a value in kilowatt_hours_per_pound.

        

        :param meters: The SpecificEnergy value in kilowatt_hours_per_pound.
        :type kilowatt_hours_per_pound: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(kilowatt_hours_per_pound, SpecificEnergyUnits.KilowattHourPerPound)

    
    @staticmethod
    def from_megawatt_hours_per_pound(megawatt_hours_per_pound: float):
        """
        Create a new instance of SpecificEnergy from a value in megawatt_hours_per_pound.

        

        :param meters: The SpecificEnergy value in megawatt_hours_per_pound.
        :type megawatt_hours_per_pound: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(megawatt_hours_per_pound, SpecificEnergyUnits.MegawattHourPerPound)

    
    @staticmethod
    def from_gigawatt_hours_per_pound(gigawatt_hours_per_pound: float):
        """
        Create a new instance of SpecificEnergy from a value in gigawatt_hours_per_pound.

        

        :param meters: The SpecificEnergy value in gigawatt_hours_per_pound.
        :type gigawatt_hours_per_pound: float
        :return: A new instance of SpecificEnergy.
        :rtype: SpecificEnergy
        """
        return SpecificEnergy(gigawatt_hours_per_pound, SpecificEnergyUnits.GigawattHourPerPound)

    
    @property
    def joules_per_kilogram(self) -> float:
        """
        
        """
        if self.__joules_per_kilogram != None:
            return self.__joules_per_kilogram
        self.__joules_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.JoulePerKilogram)
        return self.__joules_per_kilogram

    
    @property
    def mega_joules_per_tonne(self) -> float:
        """
        
        """
        if self.__mega_joules_per_tonne != None:
            return self.__mega_joules_per_tonne
        self.__mega_joules_per_tonne = self.__convert_from_base(SpecificEnergyUnits.MegaJoulePerTonne)
        return self.__mega_joules_per_tonne

    
    @property
    def calories_per_gram(self) -> float:
        """
        
        """
        if self.__calories_per_gram != None:
            return self.__calories_per_gram
        self.__calories_per_gram = self.__convert_from_base(SpecificEnergyUnits.CaloriePerGram)
        return self.__calories_per_gram

    
    @property
    def watt_hours_per_kilogram(self) -> float:
        """
        
        """
        if self.__watt_hours_per_kilogram != None:
            return self.__watt_hours_per_kilogram
        self.__watt_hours_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.WattHourPerKilogram)
        return self.__watt_hours_per_kilogram

    
    @property
    def watt_days_per_kilogram(self) -> float:
        """
        
        """
        if self.__watt_days_per_kilogram != None:
            return self.__watt_days_per_kilogram
        self.__watt_days_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.WattDayPerKilogram)
        return self.__watt_days_per_kilogram

    
    @property
    def watt_days_per_tonne(self) -> float:
        """
        
        """
        if self.__watt_days_per_tonne != None:
            return self.__watt_days_per_tonne
        self.__watt_days_per_tonne = self.__convert_from_base(SpecificEnergyUnits.WattDayPerTonne)
        return self.__watt_days_per_tonne

    
    @property
    def watt_days_per_short_ton(self) -> float:
        """
        
        """
        if self.__watt_days_per_short_ton != None:
            return self.__watt_days_per_short_ton
        self.__watt_days_per_short_ton = self.__convert_from_base(SpecificEnergyUnits.WattDayPerShortTon)
        return self.__watt_days_per_short_ton

    
    @property
    def watt_hours_per_pound(self) -> float:
        """
        
        """
        if self.__watt_hours_per_pound != None:
            return self.__watt_hours_per_pound
        self.__watt_hours_per_pound = self.__convert_from_base(SpecificEnergyUnits.WattHourPerPound)
        return self.__watt_hours_per_pound

    
    @property
    def btu_per_pound(self) -> float:
        """
        
        """
        if self.__btu_per_pound != None:
            return self.__btu_per_pound
        self.__btu_per_pound = self.__convert_from_base(SpecificEnergyUnits.BtuPerPound)
        return self.__btu_per_pound

    
    @property
    def kilojoules_per_kilogram(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kilogram != None:
            return self.__kilojoules_per_kilogram
        self.__kilojoules_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.KilojoulePerKilogram)
        return self.__kilojoules_per_kilogram

    
    @property
    def megajoules_per_kilogram(self) -> float:
        """
        
        """
        if self.__megajoules_per_kilogram != None:
            return self.__megajoules_per_kilogram
        self.__megajoules_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.MegajoulePerKilogram)
        return self.__megajoules_per_kilogram

    
    @property
    def kilocalories_per_gram(self) -> float:
        """
        
        """
        if self.__kilocalories_per_gram != None:
            return self.__kilocalories_per_gram
        self.__kilocalories_per_gram = self.__convert_from_base(SpecificEnergyUnits.KilocaloriePerGram)
        return self.__kilocalories_per_gram

    
    @property
    def kilowatt_hours_per_kilogram(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_kilogram != None:
            return self.__kilowatt_hours_per_kilogram
        self.__kilowatt_hours_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.KilowattHourPerKilogram)
        return self.__kilowatt_hours_per_kilogram

    
    @property
    def megawatt_hours_per_kilogram(self) -> float:
        """
        
        """
        if self.__megawatt_hours_per_kilogram != None:
            return self.__megawatt_hours_per_kilogram
        self.__megawatt_hours_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.MegawattHourPerKilogram)
        return self.__megawatt_hours_per_kilogram

    
    @property
    def gigawatt_hours_per_kilogram(self) -> float:
        """
        
        """
        if self.__gigawatt_hours_per_kilogram != None:
            return self.__gigawatt_hours_per_kilogram
        self.__gigawatt_hours_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.GigawattHourPerKilogram)
        return self.__gigawatt_hours_per_kilogram

    
    @property
    def kilowatt_days_per_kilogram(self) -> float:
        """
        
        """
        if self.__kilowatt_days_per_kilogram != None:
            return self.__kilowatt_days_per_kilogram
        self.__kilowatt_days_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.KilowattDayPerKilogram)
        return self.__kilowatt_days_per_kilogram

    
    @property
    def megawatt_days_per_kilogram(self) -> float:
        """
        
        """
        if self.__megawatt_days_per_kilogram != None:
            return self.__megawatt_days_per_kilogram
        self.__megawatt_days_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.MegawattDayPerKilogram)
        return self.__megawatt_days_per_kilogram

    
    @property
    def gigawatt_days_per_kilogram(self) -> float:
        """
        
        """
        if self.__gigawatt_days_per_kilogram != None:
            return self.__gigawatt_days_per_kilogram
        self.__gigawatt_days_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.GigawattDayPerKilogram)
        return self.__gigawatt_days_per_kilogram

    
    @property
    def terawatt_days_per_kilogram(self) -> float:
        """
        
        """
        if self.__terawatt_days_per_kilogram != None:
            return self.__terawatt_days_per_kilogram
        self.__terawatt_days_per_kilogram = self.__convert_from_base(SpecificEnergyUnits.TerawattDayPerKilogram)
        return self.__terawatt_days_per_kilogram

    
    @property
    def kilowatt_days_per_tonne(self) -> float:
        """
        
        """
        if self.__kilowatt_days_per_tonne != None:
            return self.__kilowatt_days_per_tonne
        self.__kilowatt_days_per_tonne = self.__convert_from_base(SpecificEnergyUnits.KilowattDayPerTonne)
        return self.__kilowatt_days_per_tonne

    
    @property
    def megawatt_days_per_tonne(self) -> float:
        """
        
        """
        if self.__megawatt_days_per_tonne != None:
            return self.__megawatt_days_per_tonne
        self.__megawatt_days_per_tonne = self.__convert_from_base(SpecificEnergyUnits.MegawattDayPerTonne)
        return self.__megawatt_days_per_tonne

    
    @property
    def gigawatt_days_per_tonne(self) -> float:
        """
        
        """
        if self.__gigawatt_days_per_tonne != None:
            return self.__gigawatt_days_per_tonne
        self.__gigawatt_days_per_tonne = self.__convert_from_base(SpecificEnergyUnits.GigawattDayPerTonne)
        return self.__gigawatt_days_per_tonne

    
    @property
    def terawatt_days_per_tonne(self) -> float:
        """
        
        """
        if self.__terawatt_days_per_tonne != None:
            return self.__terawatt_days_per_tonne
        self.__terawatt_days_per_tonne = self.__convert_from_base(SpecificEnergyUnits.TerawattDayPerTonne)
        return self.__terawatt_days_per_tonne

    
    @property
    def kilowatt_days_per_short_ton(self) -> float:
        """
        
        """
        if self.__kilowatt_days_per_short_ton != None:
            return self.__kilowatt_days_per_short_ton
        self.__kilowatt_days_per_short_ton = self.__convert_from_base(SpecificEnergyUnits.KilowattDayPerShortTon)
        return self.__kilowatt_days_per_short_ton

    
    @property
    def megawatt_days_per_short_ton(self) -> float:
        """
        
        """
        if self.__megawatt_days_per_short_ton != None:
            return self.__megawatt_days_per_short_ton
        self.__megawatt_days_per_short_ton = self.__convert_from_base(SpecificEnergyUnits.MegawattDayPerShortTon)
        return self.__megawatt_days_per_short_ton

    
    @property
    def gigawatt_days_per_short_ton(self) -> float:
        """
        
        """
        if self.__gigawatt_days_per_short_ton != None:
            return self.__gigawatt_days_per_short_ton
        self.__gigawatt_days_per_short_ton = self.__convert_from_base(SpecificEnergyUnits.GigawattDayPerShortTon)
        return self.__gigawatt_days_per_short_ton

    
    @property
    def terawatt_days_per_short_ton(self) -> float:
        """
        
        """
        if self.__terawatt_days_per_short_ton != None:
            return self.__terawatt_days_per_short_ton
        self.__terawatt_days_per_short_ton = self.__convert_from_base(SpecificEnergyUnits.TerawattDayPerShortTon)
        return self.__terawatt_days_per_short_ton

    
    @property
    def kilowatt_hours_per_pound(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_pound != None:
            return self.__kilowatt_hours_per_pound
        self.__kilowatt_hours_per_pound = self.__convert_from_base(SpecificEnergyUnits.KilowattHourPerPound)
        return self.__kilowatt_hours_per_pound

    
    @property
    def megawatt_hours_per_pound(self) -> float:
        """
        
        """
        if self.__megawatt_hours_per_pound != None:
            return self.__megawatt_hours_per_pound
        self.__megawatt_hours_per_pound = self.__convert_from_base(SpecificEnergyUnits.MegawattHourPerPound)
        return self.__megawatt_hours_per_pound

    
    @property
    def gigawatt_hours_per_pound(self) -> float:
        """
        
        """
        if self.__gigawatt_hours_per_pound != None:
            return self.__gigawatt_hours_per_pound
        self.__gigawatt_hours_per_pound = self.__convert_from_base(SpecificEnergyUnits.GigawattHourPerPound)
        return self.__gigawatt_hours_per_pound

    
    def to_string(self, unit: SpecificEnergyUnits = SpecificEnergyUnits.JoulePerKilogram, fractional_digits: int = None) -> str:
        """
        Format the SpecificEnergy to a string.
        
        Note: the default format for SpecificEnergy is JoulePerKilogram.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the SpecificEnergy. Default is 'JoulePerKilogram'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == SpecificEnergyUnits.JoulePerKilogram:
            return f"""{super()._truncate_fraction_digits(self.joules_per_kilogram, fractional_digits)} J/kg"""
        
        if unit == SpecificEnergyUnits.MegaJoulePerTonne:
            return f"""{super()._truncate_fraction_digits(self.mega_joules_per_tonne, fractional_digits)} MJ/t"""
        
        if unit == SpecificEnergyUnits.CaloriePerGram:
            return f"""{super()._truncate_fraction_digits(self.calories_per_gram, fractional_digits)} cal/g"""
        
        if unit == SpecificEnergyUnits.WattHourPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.watt_hours_per_kilogram, fractional_digits)} Wh/kg"""
        
        if unit == SpecificEnergyUnits.WattDayPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.watt_days_per_kilogram, fractional_digits)} Wd/kg"""
        
        if unit == SpecificEnergyUnits.WattDayPerTonne:
            return f"""{super()._truncate_fraction_digits(self.watt_days_per_tonne, fractional_digits)} Wd/t"""
        
        if unit == SpecificEnergyUnits.WattDayPerShortTon:
            return f"""{super()._truncate_fraction_digits(self.watt_days_per_short_ton, fractional_digits)} Wd/ST"""
        
        if unit == SpecificEnergyUnits.WattHourPerPound:
            return f"""{super()._truncate_fraction_digits(self.watt_hours_per_pound, fractional_digits)} Wh/lbs"""
        
        if unit == SpecificEnergyUnits.BtuPerPound:
            return f"""{super()._truncate_fraction_digits(self.btu_per_pound, fractional_digits)} btu/lb"""
        
        if unit == SpecificEnergyUnits.KilojoulePerKilogram:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_kilogram, fractional_digits)} kJ/kg"""
        
        if unit == SpecificEnergyUnits.MegajoulePerKilogram:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_kilogram, fractional_digits)} MJ/kg"""
        
        if unit == SpecificEnergyUnits.KilocaloriePerGram:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_gram, fractional_digits)} kcal/g"""
        
        if unit == SpecificEnergyUnits.KilowattHourPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_hours_per_kilogram, fractional_digits)} kWh/kg"""
        
        if unit == SpecificEnergyUnits.MegawattHourPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.megawatt_hours_per_kilogram, fractional_digits)} MWh/kg"""
        
        if unit == SpecificEnergyUnits.GigawattHourPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_hours_per_kilogram, fractional_digits)} GWh/kg"""
        
        if unit == SpecificEnergyUnits.KilowattDayPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_days_per_kilogram, fractional_digits)} kWd/kg"""
        
        if unit == SpecificEnergyUnits.MegawattDayPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.megawatt_days_per_kilogram, fractional_digits)} MWd/kg"""
        
        if unit == SpecificEnergyUnits.GigawattDayPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_days_per_kilogram, fractional_digits)} GWd/kg"""
        
        if unit == SpecificEnergyUnits.TerawattDayPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.terawatt_days_per_kilogram, fractional_digits)} TWd/kg"""
        
        if unit == SpecificEnergyUnits.KilowattDayPerTonne:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_days_per_tonne, fractional_digits)} kWd/t"""
        
        if unit == SpecificEnergyUnits.MegawattDayPerTonne:
            return f"""{super()._truncate_fraction_digits(self.megawatt_days_per_tonne, fractional_digits)} MWd/t"""
        
        if unit == SpecificEnergyUnits.GigawattDayPerTonne:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_days_per_tonne, fractional_digits)} GWd/t"""
        
        if unit == SpecificEnergyUnits.TerawattDayPerTonne:
            return f"""{super()._truncate_fraction_digits(self.terawatt_days_per_tonne, fractional_digits)} TWd/t"""
        
        if unit == SpecificEnergyUnits.KilowattDayPerShortTon:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_days_per_short_ton, fractional_digits)} kWd/ST"""
        
        if unit == SpecificEnergyUnits.MegawattDayPerShortTon:
            return f"""{super()._truncate_fraction_digits(self.megawatt_days_per_short_ton, fractional_digits)} MWd/ST"""
        
        if unit == SpecificEnergyUnits.GigawattDayPerShortTon:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_days_per_short_ton, fractional_digits)} GWd/ST"""
        
        if unit == SpecificEnergyUnits.TerawattDayPerShortTon:
            return f"""{super()._truncate_fraction_digits(self.terawatt_days_per_short_ton, fractional_digits)} TWd/ST"""
        
        if unit == SpecificEnergyUnits.KilowattHourPerPound:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_hours_per_pound, fractional_digits)} kWh/lbs"""
        
        if unit == SpecificEnergyUnits.MegawattHourPerPound:
            return f"""{super()._truncate_fraction_digits(self.megawatt_hours_per_pound, fractional_digits)} MWh/lbs"""
        
        if unit == SpecificEnergyUnits.GigawattHourPerPound:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_hours_per_pound, fractional_digits)} GWh/lbs"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpecificEnergyUnits = SpecificEnergyUnits.JoulePerKilogram) -> str:
        """
        Get SpecificEnergy unit abbreviation.
        Note! the default abbreviation for SpecificEnergy is JoulePerKilogram.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpecificEnergyUnits.JoulePerKilogram:
            return """J/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegaJoulePerTonne:
            return """MJ/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.CaloriePerGram:
            return """cal/g"""
        
        if unit_abbreviation == SpecificEnergyUnits.WattHourPerKilogram:
            return """Wh/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.WattDayPerKilogram:
            return """Wd/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.WattDayPerTonne:
            return """Wd/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.WattDayPerShortTon:
            return """Wd/ST"""
        
        if unit_abbreviation == SpecificEnergyUnits.WattHourPerPound:
            return """Wh/lbs"""
        
        if unit_abbreviation == SpecificEnergyUnits.BtuPerPound:
            return """btu/lb"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilojoulePerKilogram:
            return """kJ/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegajoulePerKilogram:
            return """MJ/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilocaloriePerGram:
            return """kcal/g"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilowattHourPerKilogram:
            return """kWh/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegawattHourPerKilogram:
            return """MWh/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.GigawattHourPerKilogram:
            return """GWh/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilowattDayPerKilogram:
            return """kWd/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegawattDayPerKilogram:
            return """MWd/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.GigawattDayPerKilogram:
            return """GWd/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.TerawattDayPerKilogram:
            return """TWd/kg"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilowattDayPerTonne:
            return """kWd/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegawattDayPerTonne:
            return """MWd/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.GigawattDayPerTonne:
            return """GWd/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.TerawattDayPerTonne:
            return """TWd/t"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilowattDayPerShortTon:
            return """kWd/ST"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegawattDayPerShortTon:
            return """MWd/ST"""
        
        if unit_abbreviation == SpecificEnergyUnits.GigawattDayPerShortTon:
            return """GWd/ST"""
        
        if unit_abbreviation == SpecificEnergyUnits.TerawattDayPerShortTon:
            return """TWd/ST"""
        
        if unit_abbreviation == SpecificEnergyUnits.KilowattHourPerPound:
            return """kWh/lbs"""
        
        if unit_abbreviation == SpecificEnergyUnits.MegawattHourPerPound:
            return """MWh/lbs"""
        
        if unit_abbreviation == SpecificEnergyUnits.GigawattHourPerPound:
            return """GWh/lbs"""
        